package cn.kdan.pdf.backend.core.service.impl;

import cn.kdan.pdf.backend.core.enums.ProviderEnum;
import cn.kdan.pdf.backend.core.mapper.SocialAccountsMapper;
import cn.kdan.pdf.backend.core.model.Members;
import cn.kdan.pdf.backend.core.model.SocialAccounts;
import cn.kdan.pdf.backend.core.model.SocialAccountsExample;
import cn.kdan.pdf.backend.core.params.BindAccountParam;
import cn.kdan.pdf.backend.core.pojo.third.Tencent;
import cn.kdan.pdf.backend.core.pojo.third.Wechat;
import cn.kdan.pdf.backend.core.service.SocialAccountService;
import cn.kdan.pdf.backend.core.utils.TencentUtils;
import cn.kdan.pdf.backend.core.utils.WechatUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * @author SongFuQiang
 * @description
 */
@Slf4j
@Service
public class SocialAccountServiceImpl implements SocialAccountService {

    @Resource
    private SocialAccountsMapper socialAccountsMapper;

    @Override
    public SocialAccounts selectByAccountId(String accountId) {
        SocialAccountsExample example = new SocialAccountsExample();
        example.createCriteria().andAccountIdEqualTo(accountId);
        List<SocialAccounts> list = socialAccountsMapper.selectByExample(example);
        if (!CollectionUtils.isEmpty(list)) {
            return list.get(0);
        }
        return null;
    }

    @Override
    public int insertSocialAccount(SocialAccounts socialAccounts) {
        Date date = new Date();
        socialAccounts.setCreatedAt(date);
        socialAccounts.setUpdatedAt(date);
        socialAccountsMapper.insert(socialAccounts);
        return 0;
    }

    @Override
    public SocialAccounts selectByMemberId(String userId, Integer provider) {
        SocialAccountsExample example = new SocialAccountsExample();
        example.createCriteria().andMemberIdEqualTo(userId).andProviderEqualTo(provider);
        List<SocialAccounts> list = socialAccountsMapper.selectByExample(example);
        if(CollectionUtils.isEmpty(list)){
            return null;
        }
        return list.get(0);
    }

    @Override
    public void deleteByMemberId(String memberId) {
        SocialAccountsExample example = new SocialAccountsExample();
        example.createCriteria().andMemberIdEqualTo(memberId);
        socialAccountsMapper.deleteByExample(example);
    }

    @Override
    public String bindSocialAccount(Members member, BindAccountParam param) {
        String accountId = "";
        String accessToken = param.getAccessToken();
        String openid = param.getOpenid();
        String provider = param.getProvider();
        if(ProviderEnum.TENCENT.getName().equals(provider)){
            Tencent userInfo = TencentUtils.getUserInfo(accessToken);
            String unionId = userInfo.getUnionId();
            accountId = unionId;
            SocialAccounts socialAccount = selectByAccountId(unionId);
            //看看有没有该账号，没有就新增一条，有就更新用户id
            if(ObjectUtils.isEmpty(socialAccount)){
                socialAccount.setAccountId(unionId);
                socialAccount.setMemberId(member.getId());
                socialAccount.setName(userInfo.getNickname());
                socialAccount.setProvider(ProviderEnum.getCodeByName(provider));
                socialAccount.setAvatar(userInfo.getFigureUrl());
                insertSocialAccount(socialAccount);
            } else{
                socialAccount.setMemberId(member.getId());
                updateSocialAccount(socialAccount);
            }
        }else{
            Wechat userInfo = WechatUtils.getUserInfo(openid, accessToken);
            String unionId = userInfo.getUnionId();
            accountId = unionId;
            SocialAccounts socialAccount = selectByAccountId(unionId);
            if(ObjectUtils.isEmpty(socialAccount)){
                socialAccount.setAvatar(userInfo.getHeadImgUrl());
                socialAccount.setAccountId(unionId);
                socialAccount.setProvider(ProviderEnum.getCodeByName(provider));
                socialAccount.setMemberId(member.getId());
                socialAccount.setName(userInfo.getNickname());
                insertSocialAccount(socialAccount);
            } else{
                socialAccount.setMemberId(member.getId());
                updateSocialAccount(socialAccount);
            }
        }
        return accountId;
    }

    /**
     * 根据accountId更新社交账号
     * @param socialAccount 社交账号信息
     */
    public void updateSocialAccount(SocialAccounts socialAccount) {
        SocialAccountsExample example = new SocialAccountsExample();
        example.createCriteria().andAccountIdEqualTo(socialAccount.getAccountId());
        socialAccount.setUpdatedAt(new Date());
        socialAccountsMapper.updateByExample(socialAccount,example);
    }

    @Override
    public String unbindSocialAccount(Members currentUser, BindAccountParam param) {
        String accountId = "";
        String accessToken = param.getAccessToken();
        String openid = param.getOpenid();
        String provider = param.getProvider();
        SocialAccountsExample example = new SocialAccountsExample();
        if(ProviderEnum.TENCENT.getName().equals(provider)){
            Tencent userInfo = TencentUtils.getUserInfo(accessToken);
            String unionId = userInfo.getUnionId();
            accountId = unionId;
            example.createCriteria().andAccountIdEqualTo(unionId);
            socialAccountsMapper.deleteByExample(example);
        } else {
            Wechat userInfo = WechatUtils.getUserInfo(openid, accessToken);
            String unionId = userInfo.getUnionId();
            accountId = unionId;
            SocialAccounts socialAccount = selectByAccountId(unionId);
            example.createCriteria().andAccountIdEqualTo(unionId);
            socialAccountsMapper.deleteByExample(example);
        }
        return accountId;
    }
}
